%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% DFBAlab: Dynamic Flux Balance Analysis laboratory                       %
% Process Systems Engineering Laboratory, Cambridge, MA, USA              %
% July 2014                                                               %
% Written by Jose A. Gomez and Kai Hffner                                %
%                                                                         % 
% This code can only be used for academic purposes. When using this code  %
% please cite:                                                            %
%                                                                         %
% Gomez, J.A., Hffner, K. and Barton, P. I.                              %
% DFBAlab: A fast and reliable MATLAB code for Dynamic Flux Balance       %
% Analysis. Submitted.                                                    %
%                                                                         %
% COPYRIGHT (C) 2014 MASSACHUSETTS INSTITUTE OF TECHNOLOGY                %
%                                                                         %
% Read the LICENSE.txt file for more details.                             %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

function [lb,ub] = RHS(t,y,INFO)

% This subroutine updates the upper and lower bounds for the fluxes in the
% exID arrays in main. The output should be two matrices, lb and ub. The lb matrix
% contains the lower bounds for exID{i} in the ith row in the same order as
% exID. The same is true for the upper bounds in the ub matrix.
% Infinity can be used for unconstrained variables, however, it should be 
% fixed for all time. 

N = INFO.N;
ns = INFO.ns;
nspecies = INFO.nspecies;
neflux = INFO.neflux;
eflag = INFO.eflag;
edir = INFO.edir;
DB = INFO.DB;
vmax = INFO.vmax;
km = INFO.km;

for i = 1:nspecies
    
    for j = 1:N+1
        
        % Growth rate flux bounds
        
        lb((i-1)*(N+1)+j,1) = 0;
        ub((i-1)*(N+1)+j,1) = DB((i-1)*(N+1)+j);
        
        % Metabolite flux bounds
        
        yn = y(1+(j-1)*ns:ns+(j-1)*ns);
        
        for k = 1:neflux(i)-1
            
            % Uptake rate bound
            
            wmet = vmax(i,k)*max([yn(k+nspecies) 0])/(km(i,k)+yn(k+nspecies));
            
            % Bounds for "normal" species
            
            if eflag(i) == 1
                lb((i-1)*(N+1)+j,k+1) = -wmet;
                ub((i-1)*(N+1)+j,k+1) = DB((i-1)*(N+1)+j);
            
            % Bounds for "unusual" species
            
            else
                if edir(i,k+1) == -1
                    lb((i-1)*(N+1)+j,k+1) = 0;
                    ub((i-1)*(N+1)+j,k+1) = wmet;
                else
                    lb((i-1)*(N+1)+j,k+1) = 0;
                    ub((i-1)*(N+1)+j,k+1) = DB((i-1)*(N+1)+j);
                end
             end
        end
    end
end

end
